#!/usr/bin/perl
# ------------------------------------------------------------------------------
#   ident "@(#)nmen_diskless.pl 1.4 02/01/30 SMI"
# ------------------------------------------------------------------------------
#
#  Copyright 2001 Sun Microsystems, Inc. All rights reserved.
#
# ------------------------------------------------------------------------------
package main;

BEGIN {
	require "$ENV{NHINSTALL_LIB}/tools/lib/common.pl";
}

#------------------------------------------------------------------------------
#
# end_vice_master_diskless_env :
# - copy /etc/ethers from master to this node
# - complete /etc/bootparmas with diskless entries
#------------------------------------------------------------------------------

sub end_vice_master_diskless_env {

   my ($cluster,$subnet0,$subnet1,$master_node, $vice_master_node,$server_node) = @_;

   my $vice_master_name = build_node_name($vice_master_node);
   my $master_name= build_node_name($master_node);

   my $vice_master_ip ="${subnet0}.${cluster}.${subnet1}.${vice_master_node}";
   my $master_ip ="${subnet0}.${cluster}.${subnet1}.${master_node}";

   my $local_bootparams ="$WORKING_DIR/bootparams_${master_name}";
   my $remote_bootparams = "/etc/bootparams";

   my $ethers_file ="/etc/ethers";

   if (scalar($NODES_LIST) > 2) {
     # if there is no diskless defined, the file doesn't exist
     &remote_exec("$vice_master_ip", "$RRCP ${master_ip}:${ethers_file} ${ethers_file}" ) ||
       &error("Cannot copy remotely on ${vice_master_ip} the ${ethers_file} from ${master_ip}");
   }

   # get localy from master node the /etc/bootparams file and update it to have 
   # second node id in it.
   
   &exec_cmd("$RCP ${master_ip}:${remote_bootparams} ${local_bootparams}") ||
     &error("Cannot copy ${remote_bootparams} from ${master_ip}");
   
   open(BOOTPARAMS,"${local_bootparams}") || &error("Unable to create ${local_bootparams}");
   open(TMP_FILE,">${local_bootparams}.tmp") ||
     &error("Unable to create ${local_bootparams}");
   
   while ( <BOOTPARAMS> ) {	
     chop();
     s/${master_name}/${vice_master_name}/g;
     print TMP_FILE "$_\n";
   }
   close(BOOTPARAMS);
   close(TMP_FILE);
   
   #
   # remote copy bootparams to second node
   #
   
   &exec_cmd ("$RCP ${local_bootparams}.tmp ${vice_master_ip}:${remote_bootparams}") ||
     &error ( "Unable to remote copy ${local_bootparams}.tmp in ${vice_master_ip}:${remote_bootparams}");
   
}

#-------------------------------------------------------------------------------
#
# Declare Solaris diskless client on NODE1
#
#-------------------------------------------------------------------------------
sub smdiskless {

   my ($cluster,$subnet0,$subnet1,$proto_node,$server_node) = @_;
   my $proto_name = build_node_name($proto_node);
   my $server_name= build_server_name();
   my $proto_ip   ="${subnet0}.${cluster}.${subnet1}.${proto_node}";
	
   my $server_ip  ="${subnet0}.${cluster}.${subnet1}.${server_node}";
   my $node_ip_template;
   my $node_eth_template;
   my $node_eth;
   my $node_name_template;
   my $cmd;
   my $node_ip_MEN2;
   my $i=1;
   my $crfs_file="/etc/opt/SUNWcgha/crfs.share";
   my $dfstab_file="/etc/dfs/dfstab";
   my $setting_file="/etc/default/init" ;

   #
   # We issue the smdiskless for all diskless nodes
   #
   foreach $node_count (@NODES_LIST) {
		
     $node_ip_template = "${subnet0}.${cluster}.${subnet1}.${node_count}";
     $node_eth_template = "ETH_NODE$i";
     $node_eth=$$node_eth_template;
     if ($node_count ne $NODE1 && $node_count ne $NODE2) {
       $node_name_template = build_node_name($node_count);

       print_subaction("adding $node_name_template") ;

       $cmd = "$RSMDISKLESS add -p sunrules -- -i ${node_ip_template} -e $node_eth -n ${node_name_template} -x os=sparc.${ARCH}.Solaris_8";													
       &remote_exec ( "$proto_ip", $cmd) ||
	 &error( " Cannot remotely execute $cmd" );

       # we copy the /etc/default/init to set the timezone and other variables
       &remote_exec ( "$proto_ip", "$RCOPY $setting_file /export/root/${node_name_template}${setting_file}" ) || 
	 &error ( "Unable to cp $setting_file into /export/root/${node_name_template}${setting_file}" );
					
     }
     $i++;
   }

   # Then we update crfs.share to make diskless directories managed by crfs
   # In fact we copy dfstab into crfs.share
   #
   &remote_exec ( "$proto_ip", "$RCOPY $dfstab_file $crfs_file" ) || 
     &error ( "Unable to cp $dfstab_file into $crfs_file on $proto_name" );
	
   #
   # dfstab from the second is copied on the installation node
   # this avoid same lines to appear in dfstab and crfs.share on the installation node
   #
   $node_ip_MEN2 = "${subnet0}.${cluster}.${subnet1}.${NODE2}";

   &remote_exec ( "$proto_ip", "$RRCP ${node_ip_MEN2}:$dfstab_file $dfstab_file" ) || 
     &error ( "Unable to rcp ${node_ip_MEN2}:$dfstab_file  $dfstab_file " );

   #
   # We update the NODE2 crfs.share
   #

   &remote_exec ( "$proto_ip", "$RRCP $crfs_file ${node_ip_MEN2}:$crfs_file") ||
     &error ( "Unable to rcp $crfs_file ${node_ip_MEN2}:$crfs_file" );
}

#-------------------------------------------------------------------------------
#
# Clean up the diskless environment
#
#-------------------------------------------------------------------------------
sub smdiskless_cleanup {

   my ($node_id) = @_;
   my $node_ip   ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${node_id}";

   #
   # remove root and swap directory for each diksless
   #
   foreach $node_count (@NODES_LIST) {
     if ($node_count ne $NODE1 && $node_count ne $NODE2) {

       $node_name_template = build_node_name($node_count);

       print_subaction("cleaning $node_name_template diskless environment") ;
													
       remote_exec ( "$node_ip", "$RRM -Rf /export/root/$node_name_template") ;
       remote_exec ( "$node_ip", "$RRM -Rf /export/swap/$node_name_template") ;
		    	
     }
   }
}

##################################################################################
#
#                               This is the main
#
##################################################################################

init_and_read_arg($0,
		  [ NODE_ROLE, WORKING_DIR],
		  []);


if ($RECOVERY ne "") {
  if ($NODE_ROLE eq 1) {
    print_recovery_node("Diskless environment clean-up") ;
    smdiskless_cleanup ($NODE) ;
  }
} else {
  if  ( $NODE_ROLE eq 1 ) {
    print_stage_node("Diskless environment creation") ;
    smdiskless($CLUSTER, $SUBNET0, $SUBNET1, $NODE , $SERVER_NODE);
  } else {
    
    # Complete the diskless environment on the second eligible master node
    # copy /etc/ethers from NODE1
    print_stage_node("Completing the vice-master node diskless environment ");
    end_vice_master_diskless_env($CLUSTER, $SUBNET0, $SUBNET1, $NODE1, $NODE , $SERVER_NODE);
  }
}

# That's all folk!
exit(0);







