#!/usr/bin/perl
# ------------------------------------------------------------------------------
#   ident "@(#)addon.pl 1.1 02/02/07 SMI"
# ------------------------------------------------------------------------------
#
#  Copyright 2001 Sun Microsystems, Inc. All rights reserved.
#
# ------------------------------------------------------------------------------
package main;

BEGIN {
	require "$ENV{NHINSTALL_LIB}/tools/lib/common.pl";
}


#
# What and when ?
#
#
# Packages and patches installed during a call to this script depend on:
# . the phase assigned to the package or the patch.
# . the scope of the package
# . the current phase
#
# Herefafter actions performed:
#
# Scope LOCAL:
#
# S (after Solaris)
# . installation of elements for MEN if current phase is S_MEN
# . installation of elements for NMEN if node=1, if current phase is S_NMEN
# F (after Foundation Services)
# . installation of elements for MEN if current phase is F_MEN
# . installation of elements for NMEN if node=1, if current phase is F_NMEN
# D (after Foundation Services on diskless)
# . installation of elements for MEN for both nodes and for NMEN if node=1 if current phase is F_NMEN
# A (after Application Services)
# . installation of elements for MEN if current phase is A_MEN and node is not master node reference
#   explanation:
#   . if node is the master node reference:
#     means that the script is called to install shared element: do not install local elements (see scope SHARED)
#   . if node is 1 or 2:
#     installed local elements.
# . installation of elements for NMEN if current phase is A_NMEN
#
# Scope SHARED 
#
# F
# . installation of elements if current phase is F_MEN and node=1
# A
# . installation of element if current phase is A_MEN and node is master node reference

my $FOR_MEN="";
my $FOR_NMEN="";
my $WHERE="";

#
#
#-----------------------------------------------------------
#
#   determine if the patch must be installed
#
#-----------------------------------------------------------

sub to_be_processed
  {
    my ($phase, $men, $nmen, $scope) = @_;

    my $installed = FALSE ;

    $FOR_MEN = FALSE ;
    $FOR_NMEN = FALSE ;

    if ($scope eq "SHARED") {
      if (($phase eq "F") && ($PHASE eq "F_MEN") && ($NODE eq $NODE1)) {
	$FOR_MEN = TRUE;
	return TRUE ;
      } 
      if (($phase eq "A") && ($PHASE eq "A_MEN") && ($NODE eq $NODE_MASTER)) {
	$FOR_MEN = TRUE;
	return TRUE;
      }
    }


  SWITCH:
      {
	if ($PHASE eq "S_MEN") {
	  if (($phase eq "S") && ($men eq "Y")) {
	    $installed = TRUE ;
	    $FOR_MEN = TRUE;
	  }
	  last SWITCH;
	}
	
	if ($PHASE eq "F_MEN") {
	  if (($phase eq "F") && ($men eq "Y")) {
	    $installed = TRUE ;
	    $FOR_MEN = TRUE;
	  }
	  last SWITCH;
	}
	
	if ($PHASE eq "S_NMEN") {
	  # only on node 1 (master during foundation services installation
	  if (($phase eq "S") && ($nmen eq "Y") && ($NODE eq $NODE1)) {
	    $installed = TRUE ;
	    $FOR_NMEN = TRUE;
	    $WHERE = "SOLARIS" ;
	  }
	  last SWITCH;
	}
	
	if ($PHASE eq "F_NMEN") {
	  # only on node 1 (master during foundation services installation
	  if (($phase eq "F") && ($nmen eq "Y") && ($NODE eq $NODE1)) {
	    $installed = TRUE ;
	    $FOR_NMEN = TRUE;
	    $WHERE = "DISKLESS" ;
	    last SWITCH;
	  }
	  if ($phase eq "D") {
	    $installed = TRUE ;
	    if ($men eq "Y") {
	      $FOR_MEN = TRUE;
	    }
	    if (($nmen eq "Y") && ($NODE eq $NODE1)) {
	      $FOR_NMEN = TRUE;
	      $WHERE = "DISKLESS" ;
	    }
	    last SWITCH;
	  }
	  last SWITCH;
	}
	
	if ($PHASE eq "A_MEN") {
	  if (($phase eq "A") && ($men eq "Y") && ($NODE ne $NODE_MASTER)) {
	    $installed = TRUE ;
	    $FOR_MEN = TRUE;
	  }
	  last SWITCH;
	}
	
	if ($PHASE eq "A_NMEN") {
	  # only on master node
	  if (($phase eq "A") && ($nmen eq "Y")) {
	    $installed = TRUE ;
	    $FOR_NMEN = TRUE;
	    $WHERE = "DISKLESS" ;
	  }
	  last SWITCH;
	}   
      }

    return $installed ;
  }

#-----------------------------------------------------------
#
#   installing a list of patches for MEN
#
#-----------------------------------------------------------

sub install_patches
  {
    my $node_ip    ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${NODE}";
    my $server_ip  ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${SERVER_NODE}";

    for ($i = 0 ; $i < scalar(@PATCH_LIST) ; $i++) {
      if (to_be_processed($PATCH_LIST[$i]{phase}, $PATCH_LIST[$i]{men},
			  $PATCH_LIST[$i]{nmen}, $PATCH_LIST[$i]{scope}) eq TRUE) 
	{
	  exec_cmd("$SHARE -F nfs -o ro,anon=0 $PATCH_LIST[$i]{dir}") ||
	    error ( "Unable to NFS share $SOLARIS_DIR" );   
	  if ($FOR_MEN eq TRUE) {
	    install_men_patch($NODE, $PATCH_LIST[$i]{dir}, $PATCH_LIST[$i]{subdir},
			      $PATCH_LIST[$i]{reference}, $PATCH_LIST[$i]{scope}) ;
	  } 
	  if ($FOR_NMEN eq TRUE) {
	    install_nmen_patch($NODE, $PATCH_LIST[$i]{dir}, $PATCH_LIST[$i]{subdir},
			      $PATCH_LIST[$i]{reference}, $WHERE) ;
	  }
	  exec_cmd("$UNSHARE $PATCH_LIST[$i]{dir}");
	}
    }
  }

#-----------------------------------------------------------
#
#   removing a list of patches from MEN
#
#-----------------------------------------------------------

sub remove_patches
  {
    # removed in thre reverse order
    for ($i = scalar(@PATCH_LIST) - 1 ; $i >= 0 ; $i--) {
      # patch removed: local for MEN (on both MENs) or shared (only on MEN1)
      if (to_be_processed($PATCH_LIST[$i]{phase}, $PATCH_LIST[$i]{men},
			  $PATCH_LIST[$i]{nmen}, $PATCH_LIST[$i]{scope}) eq TRUE) 
	{
	  if ($FOR_MEN eq TRUE) {
	    remove_men_patch($NODE, $PATCH_LIST[$i]{reference}, $PATCH_LIST[$i]{scope}) ;
	  } 
	  if ($FOR_NMEN eq TRUE) {
	    remove_nmen_patch($NODE, $PATCH_LIST[$i]{reference}, $WHERE) ;
	  }
	}
    }
  }

#-----------------------------------------------------------
#
#   install add-on
#
#-----------------------------------------------------------

sub install_addon
  {
    install_patches();
  }

#-----------------------------------------------------------
#
#   remove add-on
#
#-----------------------------------------------------------

sub remove_addon
  {  
    remove_patches();
  }

##################################################################################
#
#                               This is the main
#
##################################################################################

@required = [ ROOT_DIR, NODE_ROLE, WORKING_DIR, PHASE] ;
init_and_read_arg($0, @required, []) ;

if ($DO_ADDON eq FALSE) {
  exit(0) ;
}

my $name="for unknown phase";
SWITCH: {
  if ($PHASE eq "S_MEN") {
    $name = "after Solaris installation on master-eligible nodes" ;
    last SWITCH;
  }
  if ($PHASE eq "S_NMEN") {
    $name = "after Solaris installation on diskless nodes" ;
    last SWITCH;
  }
  if ($PHASE eq "F_MEN") {
    $name = "after Foundation Services installation on master-eligible nodes" ;
    last SWITCH;
  }
  if ($PHASE eq "F_NMEN") {
    $name = "after Foundation Services installation on diskless nodes" ;
    last SWITCH;
  }
  if ($PHASE eq "A_MEN") {
    $name = "after Application Services installation on master-eligible nodes" ;
    last SWITCH;
  }
  if ($PHASE eq "A_NMEN") {
    $name = "after Application Services installation on diskless nodes" ;
    last SWITCH;
  }
}

if ($RECOVERY ne "") {
  print_recovery_node("Add-on $name: clean-up") ;
  remove_addon();
  
} else {
  
  print_stage_node("Add-on $name") ;
  install_addon();
}

# That's all folk!
exit(0);





