#!/usr/bin/perl
# ------------------------------------------------------------------------------
#  ident "@(#)conf.pl 1.5 02/02/07 SMI"
# ------------------------------------------------------------------------------
#
#  Copyright 2001 Sun Microsystems, Inc. All rights reserved.
#
# ------------------------------------------------------------------------------

# ------------------------------------------------------------------------------
#
# GLOBAL VARIABLES
#
#   Configuration data used by others modules
#   Handle with care !
#
#------------------------------------------------------------------------------

# Cluster ID
$CLUSTER="";

# Node ID of the Production/Install server : IP address is then $SUBNET0.$CLUSTER.$SUBNET1.$SERVER_NODE
$SERVER_NODE="";

# Production/Install Server interface name for private network
$IFACE="";

# Architecture of machines
$ARCH=sun4u;
  
# List of nodes ID. NODE1 is the master node ID. NODE2 is the vice-master node ID (1 and 2 are reserved node Id values: do not use them)
@NODES_LIST=();
# List of Ethernet @. Follow the same order as NODE ID list
@ETH_LIST=();

# List of slices
@SLICE_LIST=();

# List of patches
$DO_ADDON=TRUE;
@PATCH_LIST=();
@PACKAGE_LIST=();

# predefined mounting point
$SLICE_ROOT="/" ;
$SLICE_SUNWCGHA="/SUNWcgha/local" ;
$SLICE_EXPORT="/export" ;
$SLICE_ROOT_MIN_SIZE=2048;
$SLICE_SUNWCGHA_MIN_SIZE=2048;
$SLICE_EXPORT_MIN_SIZE=1024 ;
$SLICE_EXPORT_MIN_SIZE_PER_DISKLESS=100;

$DIR_SUNWCGHA_EXPORTED="/SUNWcgha/local/export" ;

# network interfaces for MEN
$MEN_NIC0="hme0" ;
$MEN_NIC1="hme1" ;

# network interfaces for NMEN
$NMEN_NIC0="eri0" ;
$NMEN_NIC1="eri1" ;

# list of exported directories
@DIR_EXPORTED= ( $DIR_SUNWCGHA_EXPORTED ) ;

# list of couple (remore directory, local mounting point)
@DIR_MOUNTED=(
	      {
	       "remote" => "$DIR_SUNWCGHA_EXPORTED/data" ,
	       "local"  => "/SUNWcgha/remote"
	      } ,
	      {
	       "remote" => "$DIR_SUNWCGHA_EXPORTED/services/ha_v1/opt" ,
	       "local"  => "/SUNWcgha/services"
	      } ,
	      {
	       "remote" => "$DIR_SUNWCGHA_EXPORTED/services" ,
	       "local"  => "/SUNWcgha/swdb"
	      } ,
	     ) ;

$DISKLESS_REQUIRED = FALSE ;
# ------------------------------------------------------------------------------
#
#   Netra HAS 2.0 variables (do not change!)
#
# ------------------------------------------------------------------------------

# subnet numbers
$SUBNET0=10;
# subnet for first interface : $SUBNET0.$CLUSTER.$SUBNET1.0
$SUBNET1=1;
# subnet for second interface : $SUBNET0.$CLUSTER.$SUBNET2.0
$SUBNET2=2;
# subnet for cgtp interface : $SUBNET0.$CLUSTER.$SUBNET3.0
$SUBNET3=3;

# WORKAROUND_CMMLDAPSIMU flag
# If set to TRUE, we activate workaround CMMLDAPSIMU during install/config
$WORKAROUND_CMMLDAPSIMU="FALSE";

# WORKAROUND_DISKLESS_CGTPMASTER_MOUNT flag
# If set to TRUE, we activate workaround DISKLESS_CGTPMASTER_MOUNT during install/config
$WORKAROUND_DISKLESS_CGTPMASTER_MOUNT="FALSE";

# WORKAROUND_DISKLESS_CGTPMASTER_DHCP flag
# If set to TRUE, we activate workaround DISKLESS_CGTPMASTER_DHCP during install/config
$WORKAROUND_DISKLESS_CGTPMASTER_DHCP="FALSE";

# WORKAROUND_RBS_VICEMASTER_TFTP flag
# if set to TRUE, we activate the workaround WORKAROUND_RBS_VICEMASTER_TFTP during the install/config
$WORKAROUND_RBS_VICEMASTER_TFTP="TRUE";

# WORKAROUND_RBS_MEN_CRFS_SHARE flag
# if set to TRUE, we activate the workaround WORKAROUND_RBS_MEN_CRFS_SHARE during the install/config
# to set access and root permission to the triplet addresses of diskless nodes
$WORKAROUND_RBS_MEN_CRFS_SHARE="TRUE";

# WORKAROUND_RBSCONFIG_SCRIPT flag
# if set to TRUE, we do nothing and let the installer use rbsconfig script
$WORKAROUND_RBSCONFIG_SCRIPT="FALSE";


#
# module variables
#

$G_verbose = FALSE ;

#------------------------------------------------------------------------------
#
#  load cluster definition
#
#------------------------------------------------------------------------------

sub load_cluster
  {
    my ($filename) = @_ ;

    my $node_count = 0 ;
    
    if ($G_verbose eq TRUE) {
      print "Loading cluster definition from $filename\n" ;
    }

    open(FCLUSTER, "<$filename") or die("Can't access $filename") ;
    while (<FCLUSTER>) {
      chomp($line = $_) ;
      $line =~ s/#.*// ;
      if ($line eq "") {
	next ;
      }
      $line =~ m/[\s\t]*([^\s\t]*)[\s\t]*=[\s\t]*(.*)/ ;
      if ($line eq "") {
	next ;
      }
      if ($1 eq "CLUSTER_ID") {
	#
	#   CLUSTER_ID=<cluster-id>
	#
	if (($2 < 1) || ($2 > 254)) {
	  error("Wrong cluster Id ($2): must be between 1 and 254 (line $., file $filename)") ;
	}
	$CLUSTER=$2;
      }

      elsif ($1 eq "ARCH") {
	#
	#   ARCH=<arch>
	#
	$ARCH=$2 ;
      } 

      elsif ($1 eq "MEN_INTERFACES") {
	#
	#   MEN_INTERFACES=<nic0> <nic1>
	#
	$value = $2 ;
	$value =~ m/[\s\t]*([^\s\t]*)[\s\t]*([^\s\t]*)/;
	if ($2 eq "") {
	  error("Both interfaces must be defined for MEN nodes (line $., filename $filename)") ;
	}
	$MEN_NIC0= $1 ;
	$MEN_NIC1= $2 ;
      } 

      elsif ($1 eq "NMEN_INTERFACES") {
	#
	#   NMEN_INTERFACES=<nic0> <nic1>
	#
	$value = $2 ;
	$value =~ m/[\s\t]*([^\s\t]*)[\s\t]*([^\s\t]*)/;
	if ($2 eq "") {
	  error("Both interfaces must be defined for NMEN nodes (line $., filename $filename)") ;
	}
	$NMEN_NIC0= $1 ;
	$NMEN_NIC1= $2 ;
      } 

      elsif ($1 eq "NODE") {
	#
	# NODE=<Node id> <MAC address>
	#
	$value = $2 ;
	$value =~ m/[\s\t]*([^\s\t]*)[\s\t]*([^\s\t]*)/;
	if ($1 eq "") {
	  error("Node id is missing (line $., file $filename)") ;
	} elsif ($2 eq "") {
	  error("MAC address is missing (line $., file $filename)") ;
	}
	if (($1 < 3) || ($1 > 254)) {
	  error("Wrong node Id ($1): must be between 3 and 254 (line $., file $filename)") ;
	}
	if ($1 eq $SERVER_NODE) {
	  error("Wrong node Id ($1): already used as the installation server id (line $., file $filename)")
	}
	$node_count++ ;
	my $varname = "NODE$node_count" ;
	my $vareth  = "ETH_NODE$node_count" ;
	$$varname = $1 ;
	$$vareth = $2;
	push @NODES_LIST, $$varname ;
	push @ETH_LIST, $$vareth ;
      }

      elsif ($1 eq "EXPORTED") {
	#
	# EXPORTED=dir
	#
	if ($2 eq "") {
	  error("EXPORTED: invalid syntax (line $., file $filename)") ;
	}
	push @DIR_EXPORTED, $2 ;
      }

      elsif ($1 eq "MOUNTED") {
	#
	# MOUNTED=<local> <remote>
	#
	$value = $2 ;
	$value =~ m/[\s\t]*([^\s\t]*)[\s\t]*([^\s\t]*)/;
	if ($2 eq "") {
	  error("MOUNTED: invalid syntax (line $., file $filename)") ;
	}
	push @DIR_MOUNTED, { "remote" => "$2", "local" => "$1" } ;
      }

      elsif ($1 eq "SLICE") {
	#
	#  SLICE=<slice> <size | free> <mounting point> <bitmap slice> <options>
	#
	$value=$2 ;
	$value =~ m/[\s\t]*([^\s\t]*)[\s\t]*([^\s\t]*)[\s\t]*([^\s\t]*)[\s\t]*([^\s\t]*)[\s\t]*([^\s\t]*)/;
	if ($5 eq "") {
	  error("slice $1: invalid number of parameters (line $., file $filename)") ;
	}
	my $slice = {} ;
	$slice->{name} = $1 ;
	$slice->{size} = $2 ;
	$slice->{mounting_point} = $3 ;
	if ($4 eq "-") {
	  $slice->{bitmap} = "";
	} else {
	  $slice->{bitmap} = $4;
	}
	if ($5 eq "-") {
	  $slice->{option} = "";
	} else {
	  $slice->{option} = $5;
	}
	if (($slice->{size} eq "free") && ($slice->{bitmap} ne "")) {
	  error("slice $1: exact size required for a replicated slice (bitmap slice defined)\n") ;
	}
	push @SLICE_LIST, $slice;
      }
      else {
	error("$1: unknown parameter (line $., file $filename)") ;
      }
    }

    close(FCLUSTER) ;
  }

#------------------------------------------------------------------------------
#
#  load addon definition
#
#------------------------------------------------------------------------------

sub load_addon
  {
    my ($filename) = @_ ;

    if (! -f $filename) {
      $DO_ADDON = FALSE ;
      return ;
    }

    $DO_ADDON = TRUE;

    if ($G_verbose eq TRUE) {
      print "Loading Add-on definition from $filename\n" ;
    }

    open(FADDON, "<$filename") or die("Can't access $filename") ;
    while (<FADDON>) {
      chomp($line = $_) ;
      $line =~ s/#.*// ;
      if ($line eq "") {
	next ;
      }
      $line =~ m/[\s\t]*([^\s\t]*)[\s\t]*=[\s\t]*(.*)/ ;
      if ($line eq "") {
	next ;
      }

      if ($1 eq "PATCH") {
	#
	#   PATCH=<reference> <phase> <scope> <MEN> <NMEN>
	#
	$value = $2 ;
	$value =~ m/[\s\t]*([^\s\t]*)[\s\t]*([^\s\t]*)[\s\t]*([^\s\t]*)[\s\t]*([^\s\t]*)[\s\t]*([^\s\t]*)[\s\t]*([^\s\t]*)[\s\t]*([^\s\t]*)/;
	if ($5 eq "") {
	  error("patch $1: invalid number of parameters (line $., file $filename)") ;
	}
	my $patch = {} ;
	$patch->{reference} = $1 ;
	$patch->{dir} = $2 ;
	$patch->{subdir} = $3 ;
	if ($3 eq "-") {
	  $patch->{subdir} = "";
	}
	$patch->{phase} = $4 ;
	$patch->{scope} = $5 ;
	$patch->{men} = $6 ;
	$patch->{nmen} = $7;
	if (($patch->{phase} ne "F") && ($patch->{phase} ne "D")&& ($patch->{phase} ne "S") && ($patch->{phase} ne "A")) {
	  error("patch $1: phase must be S, F, D or A") ;
	}
	if (($patch->{scope} ne "LOCAL") && ($patch->{scope} ne "SHARED")) {
	  error("patch $1: scope must be LOCAL or SHARED") ;
	}
	if ($patch->{scope} eq "LOCAL") {
	  if (($patch->{men} ne "Y") && ($patch->{men} ne "N")) {
	    error("patch $1: MEN must be Y or N") ;
	  }
	  if (($patch->{nmen} ne "Y") && ($patch->{nmen} ne "N")) {
	    error("patch $1: NMEN must be Y or N") ;
	  }
	}
	push @PATCH_LIST, $patch;
      }
      else {
	error("$1: unknown parameter (line $., file $filename)") ;
      }
    }

    close(FADDON) ;
  }

#------------------------------------------------------------------------------
#
#  check cluster configuration
#
#------------------------------------------------------------------------------

sub check_cluster
  {
    if ($CLUSTER eq "") {
      error("CLUSTER_ID not defined") ;
    }
  }

#------------------------------------------------------------------------------
#
#  load environment variables used for configuration
#
#------------------------------------------------------------------------------

sub load_variables
  {
    $IFACE = $ENV{SERVER_INTERFACE} ;
    if ($IFACE eq "") {
      error("SERVER_INTERFACE (network interface used to access cluster) is not defined") ;
      }

    $SERVER_NODE = $ENV{SERVER_NODE} ;
    if ($SERVER_NODE eq "") {
      error("SERVER_NODE (node id of the installation server) is not defined") ;
      }

    if (($SERVER_NODE < 3) || ($SERVER_NODE > 254)) {
      error("Wrong SERVER_NODE value ($SERVER_NODE): must be between 3 and 254") ;
    }
  }

#------------------------------------------------------------------------------
#
#  look for a slice with a particular attribute
#
#------------------------------------------------------------------------------

sub slice_search
  {
    my ($attribute, $value, $idx) = @_ ;

    for $i (0..$#SLICE_LIST) {
      if ($SLICE_LIST[$i]{$attribute} eq $value) {
	$$idx = $i ;
	return 1;
      }
    }

    return 0;
  }

#------------------------------------------------------------------------------
#
#  check slice configuration
#
#------------------------------------------------------------------------------

sub check_slice
  {
    my $idx ;
    
    if (slice_search(mounting_point, $SLICE_ROOT, \$idx) != 1) {
      error("A slice with $SLICE_ROOT (root) as mounting point is required") ;
    }

    if ($SLICE_LIST[$idx]{size} < $SLICE_ROOT_MIN_SIZE) {
      error("Minimum size for $SLICE_ROOT (root) must be $SLICE_ROOT_MIN_SIZE") ;
    }

    if (slice_search(mounting_point, $SLICE_SUNWCGHA, \$idx) != 1) {
      error("A slice with $SLICE_SUNWCGHA as mounting point is required") ;
    }

    if ($SLICE_LIST[$idx]{size} < $SLICE_SUNWCGHA_MIN_SIZE) {
      error("Minimum size for $SLICE_SUNWCGHA must be $SLICE_SUNWCGHA_MIN_SIZE") ;
    }
    
    # if the user ask for no diskless, SLICE_EXPORT is not mandatory
    if ((scalar(@NODES_LIST) > 2) || ($DISKLESS_REQUIRED eq TRUE)) {
      if (slice_search(mounting_point, $SLICE_EXPORT, \$idx) != 1) {
	error ("A slice with $SLICE_EXPORT as mounting point is required") ;
      }
      my $DISKLESS_NUMBER = scalar(@NODES_LIST) - 2;
      my $MIN_SIZE = $SLICE_EXPORT_MIN_SIZE + $DISKLESS_NUMBER * $SLICE_EXPORT_MIN_SIZE_PER_DISKLESS;
      if ($SLICE_LIST[$idx]{size} < $SLICE_EXPORT_MIN_SIZE) {
	error("Minimum size for $SLICE_EXPORT must be $MIN_SIZE") ;
      }
    }
  }

#------------------------------------------------------------------------------
#
#  check exported directories
#
#------------------------------------------------------------------------------

sub check_exported
  {
    #
    # each exported directory must be a child of a replicated partition
    #
    for $i (0..$#DIR_EXPORTED) {
      my $found = FALSE ;
      for $j (0..$#SLICE_LIST) {
	if ($SLICE_LIST[$j]{bitmap} ne "") {
	  # look for slice name as a subtring of directory name
	  # add a slash at the end of the exported dir and mounting point name
	  # if not already present to avoid problem due to similar partial name
	  # ex: exported=/SUNWcgha/local/foodoo and mounting point /SUNWcgha/local/foo
	  my $exported = $DIR_EXPORTED[$i] ;
	  if (substr($exported, length($exported)-1, 1) ne "/") {
	    $exported = $exported . "/" ;
	  }
	  my $mounting_point = $SLICE_LIST[$j]{mounting_point} ;
	  if (substr($mounting_point, length($mounting_point)-1, 1) ne "/") {
	    $mounting_point = $mounting_point . "/" ;
	  }

	  my $lg = length($mounting_point) ;
	  if (substr($exported, 0, $lg) eq $mounting_point) {
	    $found = TRUE ;
	    last ;
	  }
	}
      }
      if ($found eq FALSE) {
	error("Exported directory $DIR_EXPORTED[$i] is not located on a replicated slice") ;
      }
    }

    #
    # check that an exported directory is not the child of an already exported directory
    #
    for $i (0..$#DIR_EXPORTED) {
      my $found = FALSE ;
      for $j (0..$#DIR_EXPORTED) {
	if ($i == $j) {
	  next ;
	}
	
	my $exported1 = $DIR_EXPORTED[$i] ;
	if (substr($exported1, length($exported1)-1, 1) ne "/") {
	  $exported1 = $exported1 . "/" ;
	}
	my $exported2 = $DIR_EXPORTED[$j] ;
	if (substr($exported2, length($exported2)-1, 1) ne "/") {
	  $exported2 = $exported2 . "/" ;
	}
	
	my $lg = length($exported1) ;
	if (substr($exported2, 0, $lg) eq $exported1) {
	  $found = TRUE ;
	  last ;
	}
      }
      if ($found eq TRUE) {
	error("Exported directory $DIR_EXPORTED[$j] is child of an already exported directory ($DIR_EXPORTED[$i])") ;
      }
    }
  }

#------------------------------------------------------------------------------
#
#  check remote mounted directories
#
#------------------------------------------------------------------------------

sub check_remote
  {
    #
    # each remote mounted directory must be a child of an exported directory
    #
    for $i (0..$#DIR_MOUNTED) {
      my $found = FALSE ;
      for $j (0..$#DIR_EXPORTED) {
	  # look for exported directory name as a subtring of remote directory name
	  # add a slash at the end of the exported dir and remote directory
	  # if not already present to avoid problem due to similar partial name
	  # ex: exported=/user1 and mounting point /user12
	  my $remote = $DIR_MOUNTED[$i]{remote} ;
	  if (substr($remote, length($remote)-1, 1) ne "/") {
	    $remote = $remote . "/" ;
	  }
	  my $exported = $DIR_EXPORTED[$j] ;
	  if (substr($exported, length($exported)-1, 1) ne "/") {
	    $exported = $exported . "/" ;
	  }

	my $lg = length($exported);
	if (substr($remote, 0, $lg) eq $exported) {
	  $found = TRUE ;
	  last ;
	}
      }
      if ($found eq FALSE) {
	error("Remote directory $DIR_MOUNTED[$i]{remote} for mounting point $DIR_MOUNTED[$i]{local} is not child of an exported directory") ;
      }
    }
  }

#------------------------------------------------------------------------------
#
#  checking of the configuration
#
#------------------------------------------------------------------------------

sub common_check
  {

    if (scalar(@NODES_LIST) < 2) {
      error("At least 2 nodes must be defined (2 master eligible nodes)") ;
    }
    if (scalar(@NODES_LIST) > 8) {
      error("Maximum 8 nodes are supported (2 master elgible and 6 non-master eligible nodes") ;
    }

    check_cluster ;
    check_slice ;
    check_exported ;
    check_remote ;
  }

#------------------------------------------------------------------------------
#
#  load and ckeck configuration
#
#  used by check_conf.pl
#
#------------------------------------------------------------------------------

sub load_conf_check
  {
    $G_verbose = FALSE ;

    #
    # at this stage, the user has not forced the use of diskless environment
    # we test if /export exists only if there is at least on diskless
    #
    $DISKLESS_REQUIRED = FALSE ;

    load_variables ;
    load_cluster($ENV{NH_CLUSTER_FILE_FULLNAME}) ;
    load_addon($ENV{NH_ADDON_FILE_FULLNAME}) ;

    common_check ;
  }

#------------------------------------------------------------------------------
#
#  load configuration
#
#  used by all sources except check_conf and diskless_found.pl
#
#------------------------------------------------------------------------------

sub load_conf
  {
    $G_verbose = FALSE ;

    load_variables ;
    load_cluster($ENV{NH_CLUSTER_FILE_FULLNAME}) ;
    load_addon($ENV{NH_ADDON_FILE_FULLNAME}) ;

    #
    #
    # at this stage, if there is no diskless,
    # the user has agreed about diskless environment or not
    #
    $DISKLESS_REQUIRED = TRUE ;
    if (scalar(@NODES_LIST) <= 2) {
      if ( -f "$ENV{STAGES_DIR}/NO_DISKLESS_ENV") {
	$DISKLESS_REQUIRED = FALSE ;
      }
    }

    common_check ;
  }

#------------------------------------------------------------------------------
#
#  load_conf_simple
#
#  used by all sources except diskless_found.pl
#
#------------------------------------------------------------------------------

sub load_conf_simple
  {
    #
    # the configuration is loaded only to test the number of nodes
    # so no checking
    #
    $G_verbose = FALSE ;
    load_cluster($ENV{NH_CLUSTER_FILE_FULLNAME}) ;
    load_variables ;
  }
